<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Util\InstanceProvider;
use Util\Messages;
use Util\Utilities;
use \Teknisa\Model\Geral\Entities;
use Service\Email\EmailPasswordSupportOperator;
use Zeedhi\Framework\Log\Exception;

class Operador {

    /** @var EntityManager $entityManager */
    protected  $entityManager;
    /** @var \Teknisa\Model\Geral\Repositories\Operador */
    protected  $operadorRepository;
    /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource */
    protected  $resourceRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpPerfacessoper */
    protected  $cpPerfacessoperRepository;
    /**
     * @var \Factory\Operador
     */
    private $operadorFactory;
    /**
     * @var \Factory\Resource
     */
    private $resourceFactory;
    /**
     * @var \Factory\Parcnegocio
     */
    private $parcnegocioFactory;
    /**
     * @var \Factory\CpPerfacessoper
     */
    private $perfacessparcFactory;
    /**
     * @var \Factory\CpPerfilacesso
     */
    private $cpPerfilacessoFactory;
    private $cdClienteBase;

    public function __construct(EntityManager $entityManager,\Factory\Operador $operadorFactory,
                                \Factory\Resource $resourceFactory,\Factory\Parcnegocio $parcnegocioFactory,
                                \Factory\CpPerfacessoper $perfacessparcFactory,
                                \Factory\CpPerfilacesso $cpPerfilacessoFactory,
                                $cdClienteBase){
        $this->entityManager = $entityManager;
        $this->operadorRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador');
        $this->resourceRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        $this->cpPerfacessoperRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpPerfacessoper');
        $this->operadorFactory = $operadorFactory;
        $this->resourceFactory = $resourceFactory;
        $this->parcnegocioFactory = $parcnegocioFactory;
        $this->perfacessparcFactory = $perfacessparcFactory;
        $this->cpPerfilacessoFactory = $cpPerfilacessoFactory;
        $this->cdClienteBase = $cdClienteBase;
    }

    public function findAllOperadorRelationshipCliente($condition, $page = 0, $pageSize = 0){
        $cdCliente = Utilities::getEnvironment()->getCurrentCdCliente();
        $userId    = Utilities::getEnvironment()->getCurrentUserId();
        $filter    = Utilities::createQueryFiltered($condition);
        $finalPos  = $pageSize*$page;
        $startPos  = $pageSize == 0 ? 0 : $finalPos-($pageSize-1);
        return $this->operadorRepository->findAllOperadorRelationshipCliente($cdCliente, $userId, $filter, $startPos, $finalPos);
    }

    public function findAllOperador($condition, $page = 0, $pageSize = 0){
        $filter    = Utilities::createQueryFiltered($condition);
        $finalPos  = $pageSize*$page;
        $startPos  = $pageSize == 0 ? 0 : $finalPos-($pageSize-1);
        return $this->operadorRepository->findAllOperador(Utilities::getEnvironment()->getCurrentCdCliente(), $filter, $startPos, $finalPos);
    }

    public function findAvailableCliente($condition, $page, $pageSize){
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $cdCliente = Utilities::getEnvironment()->getCurrentCdCliente();
        $filter = Utilities::createQueryFiltered($condition);
        $userId = Utilities::getEnvironment()->getCurrentUserId();

        $operador = InstanceProvider::getRepository()->getOperador()->findOneBy(array('cdoperador' => $userId));
        if($operador->getIdopermastercp() ===  'S') {
            $clientes = $this->operadorRepository->findAvailableClienteForOperatorMaster($userId, $cdCliente, $filter, $limit, $offset);
        } else {
            $clientes = $this->operadorRepository->findAvailableCliente($userId, $cdCliente, $filter);
        }
        foreach ($clientes as &$cliente) {
            if(strlen($cliente['NRINSJURCLIE']) == 14){
                $cliente['NRINSJURCLIE'] = preg_replace('/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/', "$1.$2.$3/$4-$5",$cliente['NRINSJURCLIE']);
            }

        }
        return $clientes;

    }

    public function findAllAvailableCliente($condition){
        $filter = Utilities::createQueryFiltered($condition);
        $userId = Utilities::getEnvironment()->getCurrentUserId();
        $clientes = $this->operadorRepository->findAvailableCliente($userId, 0, $filter);
        foreach ($clientes as &$cliente) {
            if(strlen($cliente['NRINSJURCLIE']) == 14){
                $cliente['NRINSJURCLIE'] = preg_replace('/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/', "$1.$2.$3/$4-$5",$cliente['NRINSJURCLIE']);
            }

        }
        return $clientes;

    }

    public function currentCliente(){
        $cdCliente = Utilities::getEnvironment()->getCurrentCdCliente();
        return $this->operadorRepository->currentCliente($cdCliente);
    }

    public function findProfileForAddProfile($cdoperador){

        $cdCliente = Utilities::getEnvironment()->getCurrentCdCliente();
        return $this->operadorRepository->findProfileForAddProfileCP($cdCliente,$cdoperador);
    }

    public function removeProfileForAddProfile($nrperfacessparc){
        $perfacessparc = $this->cpPerfacessoperRepository->findOneBy(array('nrperfacessoper'=>$nrperfacessparc));
        $this->entityManager->remove($perfacessparc);
        $this->entityManager->flush();
    }

    public function saveProfileForAddProfile($cdoperador,$profiles){
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            foreach ($profiles as $profileId) {
                $perfacessparc = $this->cpPerfacessoperRepository->findOneBy(
                    array('cdoperador'=>$cdoperador,'nrperfilacesso'=>$profileId));
                if(!$perfacessparc){
                    $perfacessparc = $this->perfacessparcFactory->create($cdoperador,$profileId);
                    $this->entityManager->persist($perfacessparc);
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveCurrentCliente($cdcliente){
        $cliente = $this->operadorRepository->currentCliente($cdcliente);
        if(!empty($cliente)){
            Utilities::getEnvironment()->setCdClienteCurrent($cdcliente);
            $client = InstanceProvider::getRepository()->getCliente()->findOneBy(array('cdcliente' => $cliente[0]['CDCLIENTE']));
            $organizacao = InstanceProvider::getRepository()->getLocalOrganizacao()->getOrganizacao($client->getNrparcnegocio());

            $cliente[0]['CLIENTE_NRPARCNEGOCIO'] = $client->getNrparcnegocio();
            $cliente[0]['CLIENTE_NRORG'] = $organizacao['NRORG'];
            return $cliente;
        } else {
            throw new CustomException('ERR_INVALID_ORGANIZATION');
        }
    }

    public function setNmacesso(Entities\Operador $operador){
        $nmacesso = $operador->getNmacesso();
        if (empty($nmacesso)) {
            $nmacesso = strtolower(substr($operador->getCdoperador().preg_replace('/[^a-zA-Z0-9_]/', "",$operador->getDsemailoper()),0,30));
            $operador->setNmacesso($nmacesso);
            $this->entityManager->persist($operador);
        }
    }

    public function currentClientSupportOperator()
    {
        $operatorRepository = InstanceProvider::getRepository()->getOperador();
        $nrorg = Utilities::getEnvironment()->getCurrentOrganizationId();
        $userId = Utilities::getEnvironment()->getCurrentUserId();
        $supportOperator = $operatorRepository->getDataSupportOperator($userId, $nrorg);
        $currentDate = new \DateTime();
        $passwordValidate = \DateTime::createFromFormat('d/m/Y H:i:s', $supportOperator['DTLIMACESSOSUP']);
        if ($passwordValidate > $currentDate) {
            $data =  InstanceProvider::getRepository()->getOperador()->getCurrentClientSupportOperator($supportOperator['NRORGSUP'], $supportOperator['CDOPERADOR']);
        } else {
            $operatorRepository->updateSupportOperator($userId, $nrorg, null, null, null);
            $data =  [];
        }
        return empty($data) ? $data : [$data];

    }

    public function saveCurrentClientSupportOperator($cdCliente)
    {
        $operatorRepository = InstanceProvider::getRepository()->getOperador();
        $nrorg = Utilities::getEnvironment()->getCurrentOrganizationId();
        $userId = Utilities::getEnvironment()->getCurrentUserId();

        if($cdCliente) {
            /** @var \Teknisa\Model\Geral\Entities\Operador $operador */
            $operador =  $operatorRepository->findOneBy(array('cdoperador' => $userId));

            /** @var \Teknisa\Model\Geral\Entities\Cliente $cliente */
            $cliente = InstanceProvider::getRepository()->getCliente()->findOneBy(array('cdcliente' => $cdCliente));
            $organizacao = InstanceProvider::getRepository()->getLocalOrganizacao()->getOrganizacao($cliente->getNrparcnegocio());

            $date = new \DateTime();
            $time = Utilities::getSupportOperatorPasswordExpiryTime();
            $date->add(new \DateInterval("PT" . $time . "M"));
            $password = $this->geraSenhaAleatoria();
            $passwordMd5 = md5($password);

            $operatorRepository->updateSupportOperator($userId, $nrorg, $passwordMd5, $date, $organizacao['NRORG']);
            $this->saveLogopera($operador->getCdoperador(), $operador->getNmoperador(), $nrorg, $organizacao['NRORG'], $date);

            $supportOperator = $operatorRepository->getDataSupportOperator($userId, $nrorg);
            $expirationDate = str_replace(' ', ' às ', $supportOperator['DTLIMACESSOSUP']);
            $expirationDate = str_replace('/', ' / ', $expirationDate);
            $expirationDate = substr($expirationDate, 0, -3);

            $data = InstanceProvider::getRepository()->getOperador()->getCurrentClientSupportOperator($organizacao['NRORG'], $supportOperator['CDOPERADOR']);
            try {
                EmailPasswordSupportOperator::sendMail($userId, $operador->getNmoperador(), $operador->getDsemailoper(), $password, $organizacao['NMORGANIZACAO'], $expirationDate);
            } catch (\Exception $e) {
                $search = array('<', '>');
                $errorMessage = str_replace($search, "", $e->getMessage());
                throw new \Exception(Messages::ERR_SEND_EMAIL_PASSWORD_SUPPORT_OPERATOR . $errorMessage);
            }
        } else {
            $operatorRepository->updateSupportOperator($userId, $nrorg, null, null, null);
            $data = [];
        }
        return empty($data) ? $data : [$data];
    }

    public function geraSenhaAleatoria()
    {
        $tamanhoSenha = 8;
        $caracteres = 'abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890#$';
        $quantidadeCaracteres = strlen($caracteres);
        $senhaGerada = '';
        for($i = 1; $i <= $tamanhoSenha; $i++){
            $rand = mt_rand(1, $quantidadeCaracteres);
            $senhaGerada .= $caracteres[$rand-1];
        }
        return 'TekOperSupp_' . $senhaGerada;
    }

    private function saveLogopera($cdoperador, $nmoperador, $nrorg, $nrorgCliente, $passwordValidate)
    {
        $tableName = 'OPERADOR';
        $currentDate = new \DateTime();
        $currentDate = $currentDate->format('d/m/Y H:i:s');
        $idtipooper = 2;
        $dsoperacao = $nrorgCliente . '=' . $passwordValidate->format('d/m/Y H:i:s');
        $dsrdmusuoper = 'VIA BD';
        $dsendusuoper = '';
        $dsnmusuoper = 'VIA BD';
        $dsmetusuoper = 'CREATE_ACCESS_SUPPORT_OPERATOR';
        $cdfillogoper = null;
        $idativo = 'S';
        InstanceProvider::getRepository()->getOperador()->saveLogopera($tableName, $currentDate, $nmoperador, $idtipooper,
                                    $dsoperacao, $dsrdmusuoper, $dsendusuoper, $dsnmusuoper, $dsmetusuoper,
                                    $cdoperador, $cdfillogoper, $nrorg, $idativo);
    }

    public function saveWithClientLogged($cdoperador, $nmoperador, $dsemailoper, $idativo, $password)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $operador = null;
            if ($cdoperador) {
                $operador = $this->operadorRepository->findOneBy(array('cdoperador' => $cdoperador));
            }
            $cdcliente = Utilities::getEnvironment()->getCurrentCdCliente();
            $password = md5($password);
            $operador = $this->operadorFactory->save($operador, $nmoperador, $dsemailoper, $idativo, $cdcliente, $password);
            $this->entityManager->persist($operador);
            $this->setNmacesso($operador);
            $this->entityManager->flush();
            if (!$cdoperador) {
                /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
                $resource = $this->resourceRepository->findOneBy(array('externalCode' => $operador->getCdoperador()));
                $resourceSponsor = $this->resourceRepository->findOneBy(array('externalCode' => $cdcliente));
                $pmStatus = Utilities::getPmStatusByIdAtivo($idativo);
                $resource = $this->resourceFactory->save($resource, $nmoperador, $dsemailoper, $pmStatus, $resourceSponsor,
                    $operador);

                $this->entityManager->persist($resource);
                $parcnegocio = $this->parcnegocioFactory->create($operador);
                $this->entityManager->persist($parcnegocio);
                $operador->setNrparcnegocio($parcnegocio->getNrparcnegocio());
                $this->setIdopersaasNotEntities($operador);
            }
            $connection->commit();
            return $operador->getCdoperador();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function findClientForSupportOperator($condition, $page, $pageSize){
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filter = Utilities::createQueryFiltered($condition);
        $userId = Utilities::getEnvironment()->getCurrentUserId();
        /** @var \Teknisa\Model\Geral\Entities\Operador $operador */
        $operador = InstanceProvider::getRepository()->getOperador()->findOneBy(array('cdoperador' => $userId));
        $cdCliente = $operador->getCdcliente();
        if($operador->getIdopermastercp() ===  'S') {
            $clientes = $this->operadorRepository->findClientsForOperatorMasterAndSupportOperator($userId, $cdCliente, $filter, $limit, $offset);
        } else {
            $clientes = $this->operadorRepository->findClientForSupportOperator($userId, $cdCliente, $filter);
        }
        foreach ($clientes as &$cliente) {
            if(strlen($cliente['NRINSJURCLIE']) == 14){
                $cliente['NRINSJURCLIE'] = preg_replace('/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/', "$1.$2.$3/$4-$5",$cliente['NRINSJURCLIE']);
            }

        }
        return $clientes;
    }



    public function changePassword($cdoperador,$password)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $operador = $this->operadorRepository->findOneBy(array('cdoperador' => $cdoperador));
            $operador->setCdsenhaoperweb(md5($password));
            $this->entityManager->persist($operador);
            $this->entityManager->flush();
            $connection->commit();
            return $operador->getCdoperador();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function setIdopersaasNotEntities(Entities\Operador $operador){

        $cdoperador = $operador->getCdoperador();
        $updateIdopersaasFromOperadorById = <<<SQL
            UPDATE OPERADOR SET IDOPERSAAS = 'N' WHERE CDOPERADOR = '$cdoperador'
SQL;
        $this->entityManager->getConnection()->executeQuery($updateIdopersaasFromOperadorById);
        $this->entityManager->flush();
    }

    public function findAllOperadorResponsible($condition, $page, $pageSize){
        $filter    = Utilities::createQueryFiltered($condition);
        $filterTeknisa = "";
        $userId = Utilities::getEnvironment()->getCurrentUserId();
        $teknisaId = $this->cdClienteBase;

        if(!Utilities::currentUserIsTeknisa()){
            $filterTeknisa = "AND OP.CDCLIENTE IN (
            SELECT CDCLIENTE
            FROM(
                SELECT CLI.CDCLIENTE
                FROM OPERCLIE OCL
                JOIN OPERADOR OPE ON (OCL.CDOPERADOR  = OPE.CDOPERADOR AND OPE.CDOPERADOR = '". $userId ."')
                JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OCL.CDCLIENTE AND CLI.IDATIVO = 'S'))
                UNION
                (SELECT CLI.CDCLIENTE
                FROM OPERADOR OPE
                JOIN CLIENTE CLI ON (CLI.CDCLIENTE = OPE.CDCLIENTE AND CLI.IDATIVO = 'S')
                WHERE OPE.CDOPERADOR = '". $userId ."')
                UNION
                SELECT '". $teknisaId ."' AS CDCLIENTE FROM DUAL
           ) ";
        }
        $finalPos  = $pageSize*$page;
        $startPos  = $pageSize == 0 ? 0 : $finalPos-($pageSize-1);
        return $this->operadorRepository->findAllOperadorResponsible($filter, $filterTeknisa, $finalPos, $startPos);
    }

}
